"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sub = void 0;
const assert_1 = __importDefault(require("assert"));
const http_1 = require("http");
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const querystring_1 = require("querystring");
const job_1 = require("../../../components/.job/job");
const BasicAuth_1 = require("./auth/BasicAuth");
class Sub extends job_1.Job {
    address = '0.0.0.0:8811';
    secure;
    auth;
    server;
    constructor({ address, secure, ...props }) {
        super(props);
        Object.assign(this, { address, secure });
        this.$$ignoreEvalProps.push('server', 'auth');
    }
    async execJob() {
        (0, assert_1.default)(this.address);
        if (this.secure?.basic) {
            this.auth = new BasicAuth_1.BasicAuth(this.secure.basic.username, this.secure.basic.password);
        }
        await new Promise((resolve, reject) => {
            const [host, port] = this.address.trim().split(':');
            this.server = (0, http_1.createServer)(async (req, res) => await this.handleRequest(req, res))
                .on('error', reject)
                .on('close', async () => {
                await super.stop();
                resolve(undefined);
            })
                .listen(+port, host, () => {
                this.logger.debug('http/jobs listened at %s', this.address);
            });
        });
    }
    async handleRequest(req, res) {
        this.logger.debug('%s %s \t%s', '⥃', req.method, req.url);
        const [path, qstr] = req.url?.split('?') || [];
        const jobData = {
            jobInfo: {
                path,
                method: req.method,
                headers: req.headers,
                query: (0, querystring_1.parse)(qstr)
            },
            jobData: {},
            jobRes: undefined
        };
        jobData.jobData = jobData.jobInfo.query;
        const logs = [this.auth ? 'secured' : 'unsecured'];
        try {
            const requestBodyText = await this.getRequestBody(req);
            const requestBody = requestBodyText ? JSON.parse(requestBodyText) : requestBodyText;
            (0, lodash_merge_1.default)(jobData.jobData, requestBody);
            const isAuth = await this.auth?.verify(jobData);
            if (isAuth === false) {
                res.statusMessage = logs.join('-');
                res.statusCode = 401;
                res.end();
                return;
            }
            if (jobData.jobInfo.query.jobRes) {
                logs.push('pipe');
                jobData.jobRes = res;
                await this.runEachOfElements(jobData);
            }
            else {
                if (!this.jobsManager) {
                    jobData.jobRes = res;
                    logs.push('pipe');
                }
                else {
                    logs.push('queue');
                }
                await this.addJobData(jobData);
            }
        }
        finally {
            res.statusCode = 200;
            res.statusMessage = logs.join('-');
            res.end();
        }
    }
    async getRequestBody(req) {
        return await new Promise((resolve, reject) => {
            const chunks = [];
            req.on('data', (chunk) => chunks.push(chunk));
            req.on('error', reject);
            req.on('end', () => {
                resolve(Buffer.concat(chunks).toString());
            });
        });
    }
    async stop() {
        if (!this.server?.listening)
            return;
        this.server?.close();
    }
    async dispose() {
        await this.stop();
        await super.dispose();
    }
}
exports.Sub = Sub;
//# sourceMappingURL=sub.js.map