"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Scene = void 0;
const assert_1 = __importDefault(require("assert"));
const chalk_1 = __importDefault(require("chalk"));
const promises_1 = require("fs/promises");
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const path_1 = require("path");
const aes_1 = require("../../libs/encrypt/aes");
const env_1 = require("../../libs/env");
const file_remote_1 = require("../../libs/file-remote");
const variable_1 = require("../../libs/variable");
const yaml_1 = require("yaml");
const element_builder_1 = require("../element-builder");
const group_1 = require("../group/group");
const constants_1 = require("./constants");
const REGEX_FIRST_UPPER = /^[A-Z]/;
class Scene extends group_1.Group {
    path;
    encryptedPath;
    content;
    password;
    curDir = '';
    localVars = {};
    isRoot = false;
    get innerScene() {
        return this;
    }
    get elementBuilder() {
        return new element_builder_1.ElementBuilder(this);
    }
    constructor({ path, encryptPath, content, password, ...props }) {
        super(props);
        this.$$ignoreEvalProps.push('content', 'curDir', 'localVars', 'elementBuilder');
        Object.assign(this, { path, encryptPath, content, password });
    }
    async asyncConstructor() {
        const remoteFileProps = await this.getRemoteFileProps();
        if (Array.isArray(remoteFileProps)) {
            this.path && this.logger.trace('%s \t%s', 'Scene', chalk_1.default.underline(this.path));
            this.lazyInit(remoteFileProps);
        }
        else {
            const { title: _title, log: _log, password: _password, vars: _vars, vars_file: _varsFile, ...groupProps } = remoteFileProps;
            const { title, log, password, vars, varsFile } = await this.getVars({ title: _title, log: _log, password: _password, vars: _vars, varsFile: _varsFile }, this);
            if (log)
                this.log = log;
            if (this.title === undefined)
                this.title = title;
            this.path && this.logger.trace('%s \t%s', 'Scene', chalk_1.default.underline(this.path));
            if (password && !this.password) {
                await this.generateEncryptedFile(this.content, password);
            }
            this.lazyInit(groupProps);
            await this.loadVars(vars, varsFile);
        }
    }
    async exec() {
        this.copyVarsToLocal();
        if (this.isRoot)
            this.logger.log(chalk_1.default.yellow('⚡️'));
        const results = await super.exec();
        return results || [];
    }
    async dispose() {
        this.copyVarsToGlobal();
        await super.dispose();
    }
    getPath(p) {
        if (!p)
            return p;
        if ((0, path_1.isAbsolute)(p))
            return p;
        if (p.startsWith('~~/'))
            return (0, path_1.join)(this.rootScene.runDir, p.substring(3));
        if (p.startsWith('~/'))
            return (0, path_1.join)(this.rootScene.rootDir, p.substring(2));
        return (0, path_1.join)(this.curDir || '', p);
    }
    async getVars(str, ctx, others = {}) {
        if (ctx?.$$loggerLevel) {
            others.parentState = ctx.parentState;
        }
        return await (0, variable_1.getVars)(str, ctx, {
            vars: this.localVars,
            utils: this.rootScene.globalUtils,
            ...others
        });
    }
    async setVars(varObj, vl, ctx) {
        return await (0, variable_1.setVars)(varObj, vl, ctx, {
            vars: this.localVars,
            utils: this.rootScene.globalUtils
        });
    }
    mergeVars(obj) {
        (0, lodash_merge_1.default)(this.localVars, obj);
    }
    copyVarsToLocal() {
        this.copyVarsToGlobal(this.scene.localVars);
        Object.assign(this.localVars, this.rootScene.globalVars);
    }
    copyVarsToGlobal(localVars = this.localVars) {
        Object.keys(localVars)
            .filter(key => REGEX_FIRST_UPPER.test(key[0]))
            .forEach(key => {
            this.rootScene.globalVars[key] = localVars[key];
        });
    }
    async getRemoteFileProps() {
        if (this.path) {
            const fileRemote = new file_remote_1.FileRemote(this.path, this.scene || this);
            this.content = await fileRemote.getTextContent();
            if (!fileRemote.isRemote) {
                this.path = (0, path_1.resolve)(fileRemote.uri);
                const dirPath = (0, path_1.dirname)(this.path);
                if (this.isRoot)
                    this.rootScene.rootDir = dirPath;
                this.curDir = dirPath;
            }
        }
        (0, assert_1.default)(this.content, 'Scene file is not valid format');
        const remoteFileProps = (0, yaml_1.parse)(await this.decryptContent(this.content, this.password));
        return remoteFileProps;
    }
    async decryptContent(content, password) {
        if (!password)
            return content;
        try {
            const encryptor = new aes_1.AES(`${constants_1.prefixPassword}${password}`);
            return encryptor.decrypt(content);
        }
        catch (err) {
            if (err?.code === 'ERR_OSSL_BAD_DECRYPT') {
                throw new Error(`Password to decrypt the file "${this.path}" is not valid`);
            }
            throw err;
        }
    }
    async generateEncryptedFile(content, password) {
        if (!password || !this.path || !content)
            return;
        this.encryptedPath = (0, path_1.join)(this.curDir, (0, path_1.basename)(this.path).split('.')[0]);
        this.logger.trace('Encrypted to\t%s', this.encryptedPath);
        const encryptor = new aes_1.AES(`${constants_1.prefixPassword}${password}`);
        const econtent = encryptor.encrypt(content);
        await (0, promises_1.writeFile)(this.encryptedPath, econtent);
    }
    async loadVars(vars = {}, varsFile) {
        if (varsFile) {
            const file = new file_remote_1.FileRemote(varsFile, this);
            const content = await file.getTextContent();
            let newVars = {};
            try {
                newVars = JSON.parse(content);
            }
            catch {
                newVars = (0, yaml_1.parse)(content);
            }
            (0, lodash_merge_1.default)(vars, newVars);
        }
        this.mergeVars(vars);
        if (this.isRoot)
            await this.loadEnv();
        if (this.vars) {
            const overridedVars = await (this.scene || this).getVars(this.vars, this);
            this.mergeVars(overridedVars);
        }
        this.logger.trace('[vars]    \t%j', this.localVars);
    }
    async loadEnv(...envFiles) {
        const env = new env_1.Env(this.logger);
        await env.loadEnvToBase(this.localVars, ...envFiles.filter(e => e), process.env);
    }
}
exports.Scene = Scene;
//# sourceMappingURL=scene.js.map