"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PackagesManager = void 0;
const assert_1 = __importDefault(require("assert"));
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const path_1 = require("path");
const exec_1 = require("./../components/exec/exec");
const logger_1 = require("./../libs/logger");
const progress_bar_1 = require("./../libs/progress-bar");
const modules_manager_1 = require("./modules-manager");
const PackageManagerSupported = ['pnpm', 'yarn', 'npm'];
class PackagesManager {
    scene;
    bin;
    logger;
    tag = 'package-manager';
    get cmdInstall() {
        if (this.bin === 'pnpm')
            return ['pnpm', 'add', '--prefer-offline', '--dir', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
        if (this.bin === 'yarn')
            return ['yarn', 'add', '--prefer-offline', '--cwd', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
        return ['npm', 'add', '--prefix', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
    }
    get cmdUpgrade() {
        if (this.bin === 'pnpm')
            return ['pnpm', 'upgrade', '--dir', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
        if (this.bin === 'yarn')
            return ['yarn', 'upgrade', '--cwd', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
        return ['npm', 'upgrade', '--prefix', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
    }
    get cmdUninstall() {
        if (this.bin === 'pnpm')
            return ['pnpm', 'remove', '--dir', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
        if (this.bin === 'yarn')
            return ['yarn', 'remove', '--cwd', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
        return ['npm', 'uninstall', '--prefix', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
    }
    static get Dependencies() {
        const packageJSON = (0, fs_1.readFileSync)((0, path_1.join)(modules_manager_1.nodeModulesDir, '../package.json')).toString();
        return JSON.parse(packageJSON).dependencies || {};
    }
    constructor(scene) {
        this.scene = scene;
        this.logger = this.scene.logger.clone(this.tag);
        this.bin = PackageManagerSupported.find((bin) => {
            try {
                const log = (0, child_process_1.execSync)(`${bin} -v`).toString();
                return !!log;
            }
            catch { }
            return false;
        });
        (0, assert_1.default)(this.bin, 'Could not found "npm" or "yarn" to install lack packages');
    }
    async exec(des, cmds, packages) {
        if (!packages.length)
            return;
        const cmd = [...cmds, ...packages];
        const msg = `tags ${packages.map(e => `"${e}"`).join(', ')}`;
        const bar = !this.logger.is(logger_1.LoggerLevel.SILENT) ? new progress_bar_1.ProgressBar(this.logger) : undefined;
        await bar?.start(`${des} ${msg}`);
        const exec = await this.scene.elementBuilder
            .element(exec_1.Exec, { commands: cmd })
            .tag(this.tag)
            .logger(this.logger)
            .build();
        try {
            await exec.exec();
            await bar?.stop();
            this.logger.info(`${des} ${msg} successfully`);
        }
        catch (err) {
            await bar?.stop();
            this.logger.error(`${des} ${msg} failed`);
            throw err;
        }
        finally {
            await exec.dispose();
        }
    }
    async install(...packages) {
        const newPackages = this.getNotInstalledPackages(...packages);
        newPackages.length && await this.exec('Installing', this.cmdInstall, newPackages);
    }
    async upgrade(...packages) {
        const existsPackages = this.getInstalledPackages(...packages);
        existsPackages.length && await this.exec('Upgrading', this.cmdUpgrade, existsPackages);
    }
    async uninstall(...packages) {
        const existsPackages = this.getInstalledPackages(...packages);
        existsPackages.length && await this.exec('Uninstalling', this.cmdUninstall, existsPackages);
    }
    async clean() {
        const modules = Object.keys(PackagesManager.Dependencies);
        await this.uninstall(...modules);
        return modules;
    }
    getInstalledPackages(...packages) {
        return packages.filter(packageName => {
            const path = (0, path_1.join)(modules_manager_1.nodeModulesDir, packageName);
            return (0, fs_1.existsSync)(path);
        });
    }
    getNotInstalledPackages(...packages) {
        return packages.filter(packageName => {
            const path = (0, path_1.join)(modules_manager_1.nodeModulesDir, packageName);
            return !(0, fs_1.existsSync)(path);
        });
    }
}
exports.PackagesManager = PackagesManager;
//# sourceMappingURL=packages-manager.js.map